-- Booked on Time!
-- "tove" is a svg implementation library created by poke1024. Link: https://github.com/poke1024/tove2d

-- Note: If this is ported to other platforms or languages, please note that changes made be made to asset implementation, and code.
-- Note: Some code may/may not be inefficient. If there is any efficency that is needed in this code for porting or whatever, please use it.

-- Dated: 2021 - 2022
local tove = require 'tove' -- Local variable to get the tove folder to implement the aforementioned svg library.


-- fixColor is a helper function that I had learned about from the Love2D fourms. You divide the numbers that make up the the color by 255, and return all of them in a table. This is due to changes in implementing color into love2D games. As of now, the numbers in the love.graphics.getColor(whatevever num, whatevernum,..,alpha)), should be decimals. Because of this, when drawing, we say love.graphics.getColor(fixColor(whatever num, whatever num,...,alpha))
function fixColor(r,g,b,a)
  nR = r/255 -- Divide first num by 255
  nG = g/255 -- Divide second number by 255
  nB = b/255 -- Divide third number by 255
  nA = a/255 -- Divide alpha (opacity number) by 255.
  
  return {nR,nG,nB,nA} -- Return it in a table, to be called in the love.graphiscs.setColor() function.
end

--function newDimensions(img,newWidth,newHeight)
  --local currentWidth, currentHeight = img:getDimensions()
  --return (newWidth / currentWidth), (newHeight / currentHeight)
--end

endWait = 3
BOOKS_GOTTEN = 0
BOOKS_NEDDED = 15
num = 40
lcdFont = love.graphics.newFont("lcd.otf",50) -- Set font for keeping time, and score.
arcoFontSmall = love.graphics.newFont("arcoSMALL.otf",20) -- Set font for keeping knowledge and books recieved
arcoFontLarge = love.graphics.newFont("ARCO.otf",50) -- Set font for larger messages
DRAWN = 1 -- Psuedo-boolean will tell us whether our player character is being drawn to the screen or not,
appear_cooldown = 0.6
--ratio1 = 1.33333333333 -- Screen ratio for 4:3 screens.
--ratio2 = 1.77777777778 -- Screen ratio for 16:9 screens.
--ratio3 = 2.33333333333 -- Screen ratio for 21:9 screens.


book_table = {} -- Create table of book (item) containers.
book_table.items = {} -- Create item container for the items, each item in this container will have it's own set of information.
misses = 0
drawI = 0

local screenState
gamestates = {"menu","s1","s2","s3","count","play","pause","win","win2","lose","lose2"} -- List of gamestates
--screenstates = {"won","lost"} -- List of screen states
book = love.filesystem.read("book.svg") -- Get book source svg
bookGFX = tove.newGraphics(book,20) -- Create tove graphics
bookGFX:setDisplay("texture") -- Set display to texture

-- Function that spawns the item (book)
function book_table:spawn(x,y,svg)
  book = {} -- Create new table.
  book.x = x -- Set the x position to x parameter
  book.y = y -- Set the y position to y parameter
  book.width = svg:getWidth() -- Book Width
  book.height = svg:getHeight() -- Book height
  book.bookDRAWN = 1 -- Psuedo-boolean will tell us whether the object is being drawn to the screen or not,
  book.bookAppearTime = 0.50 -- Create variable that determines amount of time the object will appear on screen.
  book.bookNotAppearTime = 0.50 --Amount of time that the character will not appear during position transition
  book.bookNotDrawTime = book.bookNotAppearTime  -- Creates variable called booknotDrawTime that will set it to booknotAppearTIme.
  book.hasCollided = false -- Check if book has collided with player.
  table.insert(book_table.items,book) -- Add item Book to book_table.items.
end





catchSound = love.audio.newSource("fallsound.wav", "static") -- Create sound variable that will be used whenever we catch a book.
catchSound:setLooping(false) -- Sets looping condition of moveSound to false.
timeSound = love.audio.newSource("timetick.wav","static") -- Create time tick sound 
timeSound:setLooping(false) -- Set looping condition of timeSound to false.
badCatchSound = love.audio.newSource("badpickup.wav", "static") -- Create sound variable that will be used whenever a book falls past us.
badCatchSound:setLooping(false) -- Set looping condition of badCatchSound to false

--function getRatio()
  --playRatio = love.graphics.getWidth()/love.graphics.getHeight()
  --return playRatio
--end

score = 0
miss = 0
-- Function for checking collision
function checkCollision(bookTable,colBox)
  for i,v in ipairs(bookTable) do
    if (v.x < colBox.x + colBox.width and v.x + v.width > colBox.x) then -- Bounding Box Collision 1
      if (v.y < colBox.y + colBox.height and v.y + v.height > colBox.y) then -- Bounding Box Collision 2
        score = score + 5
        BOOKS_GOTTEN = BOOKS_GOTTEN + 1 -- Increase books recieved
        table.remove(bookTable,i) -- Remove item
        catchSound:play() -- Play catch sound
      end
    end
  end
end

function checkFailedCollision(bookTable,failedBox)
  for i,v in ipairs(bookTable) do
    for _,c in pairs(failedBox) do
      if (v.x < c[1] + 32 and v.x + v.width > c[1]) then -- Bounding Box Collision 1
        if (v.y < c[2] + 24 and v.y + v.height > c[2]) then -- Bounding Box Collision 2
          v.hasCollided = true
          badCatchSound:play()
          misses = misses + 1
          drawI = drawI + 1
        end
      end
    end
  end
end

local mouseX -- Mouse/touchscreen x position.
local mouseY -- Mouse/touchscreen y position.
local mouseWidth = 1
local mouseHeight = 1

  
-- Default Love2D load function
function love.load()
  touchscreenRectangle_left = {} -- Create table for left rectangle touchbox
  touchscreenRectangle_left.x = 0 -- Create x position of left rec touchbox
  touchscreenRectangle_left.y = 0 -- Assign y position of left rec touchbox
  touchscreenRectangle_left.width = love.graphics.getWidth()/2 -- Assign width of left rec touchbox
  touchscreenRectangle_left.height = love.graphics.getHeight() -- Assign height of left rec touchbox
  
  touchscreenRectangle_right = {} -- Create table for right rectangle toucbox
  touchscreenRectangle_right.x = love.graphics.getWidth()/2 -- Assign x position of right rec touchbox
  touchscreenRectangle_right.y = 0 -- Assign y position of right rec touchbox
  touchscreenRectangle_right.width = love.graphics.getWidth()/2 -- Assign width of right rec touchbox
  touchscreenRectangle_right.height = love.graphics.getHeight() -- Asssign height of right rec touchbox
  endWait = 3 -- Wait timer when at the end of a use of a state.
  BOOKS_GOTTEN = 0 -- Books that have been gotten.
  BOOKS_NEDDED = 15 -- Initial amount of books that are needed.
  state = gamestates[6] -- Initial game state
  BEGIN_ITEM_TIMER = false
  timeTickDelay = 1
  randomX = love.math.random(1,3) -- Random position for book spawning
  math.randomseed(os.time()) -- Random number seed
  joysticks = love.joystick.getJoysticks()
  joystick = joysticks[1]
  love.joystick.loadGamepadMappings("gamecontrollerdb.txt") -- Load controller mappings
  notAppearTime = 0.50 -- Amount of time that the character will not appear during position transition
  notDrawTime = notAppearTime -- Creates variable called notDrawTime that will set it to notAppearTIme.
  randomAppear = love.math.random(2,5)
  itemAppearTime = randomAppear -- Amount of time between items appearing
  love.graphics.setDefaultFilter("nearest","nearest") -- Sets default Love2D display filter to nearest-neighbor.
  players1 = love.filesystem.read("hotplayer1.svg") -- Variable players1 that gets svg file for player character when on the left side.
  players2 = love.filesystem.read("hotplayer.svg") -- Variable "players" that gets the svg file used for the player character when in the middle.
  players3 = love.filesystem.read("hotplayer3.svg") -- Variable players3 that gets the svg file used for the player character when on the right side.
  playerGFX_left = tove.newGraphics(players1,98) -- Creates new graphics using the svg we got, and the tove function tove.newGraphics(svg,size)
  playerGFX_middle = tove.newGraphics(players2,98) -- Creates new graphics using the svg we got, and the tove function tove.newGraphics(svg,size)
  playerGFX_right = tove.newGraphics(players3,98) -- Creates new graphics using the svg we got, and the tove function tove.newGraphics(svg,size)
  fail = love.filesystem.read("fail.svg") -- Gets svg file for graphic shown when player has lost
  succeed = love.filesystem.read("success.svg") -- Gets svg file for graphic shown when player has won
  
  fail_GFX = tove.newGraphics(fail,200) -- Creates new graphics using svg we got
  success_GFX = tove.newGraphics(succeed,200) -- Creates new graphics using svg we got
  fail_GFX:setDisplay("texture") -- Set display type of tove graphic to "texture"
  success_GFX:setDisplay("texture") -- Set display type of tove graphic to "texture"
  playerGFX_left:setDisplay("texture") -- Set display type of tove graphic to "texture"
  playerGFX_middle:setDisplay("texture") -- Set display type of tove graphic to "texture"
  playerGFX_right:setDisplay("texture") -- Set display type of tove graphic to "texture"
  lcdFont:setFilter("linear","linear") -- Set display filter for lcdFont to linear
  brain = love.filesystem.read("brain.svg") -- Get nucleus svg file
  brainGFX = tove.newGraphics(brain,500)
  brainGFX:setDisplay("texture") -- Set display to texture
  platform = {} -- Create platform table.
  platform.width = love.graphics.getWidth() - 96 -- Set platform width
  platform.height = 24 -- Set platform height
  platform.x = love.graphics.getWidth() / 2 - platform.width / 2 -- Set platform x position to the center
  platform.y = love.graphics.getHeight() / 2 -- Set platform.y position to center
  

  player = {} -- Create player table
  player.width = 32 -- Set player width
  player.height = 16 -- Set player height
  
  
  -- Three tables made in the case that there were different screen sizes with ratios like 4:3, 16:9, 21:9.
  -- Each table will have three or more tables with two entries on each. Entry one will be the x position, and entry two will be the y position.
  -- Each base table like pos1 and pos2 are tables of positions for the player that will be taken if the player movies left or right.
  pos1 = {
    {0 + 96, platform.y - player.height * 2},
    {love.graphics.getWidth() / 2 - player.width /2 , platform.y - player.height * 2},
    {platform.width - player.width, platform.y - player.height * 2}
  }

  --pos2 = {
    --{0 + 96, platform.y - player.height * 2},
    --{(0 + 96) + 96 + player.width, platform.y - player.height * 2},
    --{love.graphics.getWidth() / 2 - player.width / 2 , platform.y - player.height * 2},
    --{(love.graphics.getWidth() / 2 - player.width / 2) + 96 + player.width, platform.y - player.height * 2},
    --{platform.width - player.width, platform.y - player.height * 2}
  --}

  --pos3 = {
    --{0 + 96, platform.y - player.height * 2},
    --{(0 + 96) + 96 + player.width, platform.y - player.height * 2},
    --{((0 + 96) + 96 + player.width) + 96 + player.width, platform.y - player.height * 2},
    --{love.graphics.getWidth() / 2 - player.width /2 , platform.y - player.height * 2},
    --{(love.graphics.getWidth() / 2 - player.width /2) + 96 + player.width, platform.y - player.height * 2},
    --{((love.graphics.getWidth() / 2 - player.width /2) + 96 + player.width) + 96 + player.width, platform.y - player.height * 2},
    --{platform.width - player.width, platform.y - player.height * 2}
  --}
  
  position = pos1 -- Set position var to the pos1, thus making it a table.
  player.x = position[2][1] -- Set player x position to first x position
  player.y = position[2][2] -- Set player y position to first y position.
  
  collisionRect  = {} -- Table for collision rectangle
  collisionRect.x = player.x -- Collision rectangle x position
  collisionRect.y = player.y -- Collision rectangle y position
  collisionRect.width = 64 -- Collision rectangle width
  collisionRect.height = 64 -- Collision rectangle height
  
  failCollisionBoxes = { -- Table for failed collision boxes
    {position[1][1],position[1][2] + 60}, -- Position 1
    {position[2][1],position[2][2] + 60}, -- Position 2
    {position[3][1],position[3][2] + 60}  -- Position 3
  }
  
  playerGFXtable = {playerGFX_left,playerGFX_middle,playerGFX_right} -- Player graphics table
  playerImage = playerGFXtable[2] -- Make player image equal to middle gfx.
  
  errorBox = {}
  errorBox.width = 16
  errorBox.height = 16
  
  errorBox.gfxes = {
    {(love.graphics.getWidth()/2 - errorBox.width / 2) - 30, love.graphics.getHeight() - 20},
    {(love.graphics.getWidth()/2 - errorBox.width / 2), love.graphics.getHeight() - 20},
    {(love.graphics.getWidth()/2 - errorBox.width / 2) + 30, love.graphics.getHeight() - 20}
  }
  
end

-- Default love key pressed function. This part is one of the most difficult parts of the code, so buckle up and pay attention.
-- The goal is that as the player moves left or right, the position index [i][j] will change as well. Say for example, if the player position
-- was located at position[1][1] (x position), and position[1][2] (y position). Basically, position[i] will be the index number where there is a table with two entries, and position[j] will be one of the entries, either 1 or 2.
function love.keypressed(key)
  -- When the right key is pressed.
  if (key == "right") then
    if (state == gamestates[6]) then
      if (player.x == position[1][1]) and (player.y == position[1][2]) then
        DRAWN = 0 -- Position transition will begin
        player.x = position[2][1] -- Player's x position will go right, position table index will go up by 1, leading to a new x position.
        player.y = position[2][2] -- Position table index will go up by 1, leading to a new y position.
        --moveSound:play() -- Play move sound
      elseif (player.x == position[2][1]) and (player.y == position[2][2]) then
        DRAWN = 0 -- Position transition will begin
        player.x = position[3][1] -- Position table index i will go up by 1, leading to new x position.
        player.y = position[3][2] -- Position table index i will go up by 1, leading to new y position.
        --moveSound:play() -- Play move sound
      end
      
      if ((position == pos2) or (position == pos3)) then
        if (player.x == position[3][1]) and (player.y == position[3][2]) then
          DRAWN = 0 -- Position transition will begin
          player.x = position[4][1] -- Position table index i will go up by 1, leading to a new x position
          player.y = position[4][2] -- Position table index i will go up by 1, leading to a new y position
          --moveSound:play() -- Play move sound
        elseif (player.x == position[4][1]) and (player.y == position[4][2]) then
          DRAWN = 0 -- Position transition will begin
          player.x = position[5][1] -- Position table index i will go up by 1, leading to a new x position
          player.y = position[5][2] -- Position table index i will go up by 1, leading to a new y position
          moveSound:play() -- Play move sound
        end
      end
      
      if ((position == pos3) and not(position == pos2)) then
        if (player.x == position[5][1]) and (player.y == position[5][2]) then
          DRAWN = 0 -- Position transition will begin
          player.x = position[6][1] -- Position table index i will go up by 1, leading to a new x position
          player.y = position[6][2] -- Position table index i will go up by 1, leading to a new y position
          --moveSound:play() -- Play move sound
        elseif (player.x == position[6][1]) and (player.y == position[6][2]) then
          DRAWN = 0 -- Position transition will begin
          player.x = position[7][1] -- Position table index i will go up by 1, leading to a new x position
          player.y = position[7][2] -- Position table index i will go up by 1, leading to a new y position
          --moveSound:play() -- Play move sound
        end
      end
    end
  end
  
  if (key == "left") then
    if (state == gamestates[6]) then
      if ((position == pos3) and not (position == pos2)) then
        if (player.x == position[7][1]) and (player.y == position[7][2]) then
          DRAWN = 0 -- Position transition will begin
          player.x = position[6][1] -- Position table index i will go down by 1, leading to a new x position
          player.y = position[6][2] -- Position table index i will go down by 1, leading to a new y position
          --moveSound:play() -- Play move sound
        elseif (player.x == position[6][1]) and (player.y == position[6][2]) then
          DRAWN = 0 -- Position transition will begin
          player.x = position[5][1] -- Position table index i will go down by 1, leading to a new x position
          player.y = position[5][2] -- Position table index i will go down by 1, leading to a new y position
          --moveSound:play() -- Play move sound
        end
      end
      
      if ((position == pos2) or (position == pos3)) then
        if (player.x == position[5][1]) and (player.y == position[5][2]) then
          DRAWN = 0 -- Position transition will begin
          player.x = position[4][1] -- Position table index i will go down by 1, leading to a new x position
          player.y = position[4][2] -- Position table index i will go down by 1, leading to a new y position
          --moveSound:play() -- Play move sound
        elseif (player.x == position[4][1]) and (player.y == position[4][2]) then
          DRAWN = 0 -- Position transition will begin
          player.x = position[3][1] -- Position table index i will go down by 1, leading to a new x position
          player.y = position[3][2] -- Position table index i will go down by 1, leading to a new y position
          --moveSound:play() -- Play move sound
        end
      end
      
      if (player.x == position[3][1]) and (player.y == position[3][2]) then
        DRAWN = 0 -- Position transition will begin
        player.x = position[2][1] -- Position table index i will go down by 1, leading to a new x position
        player.y = position[2][2] -- Position table index i will go down by 1, leading to a new y position
        --moveSound:play() -- Play move sound
      elseif (player.x == position[2][1]) and (player.y == position[2][2]) then
        DRAWN = 0 -- Position transition will begin
        player.x = position[1][1] -- Position table index i will go down by 1, leading to a new x position
        player.y = position[1][2] -- Position table index i will go down by 1, leading to a new y position
        --moveSound:play() -- Play move sound
      end
    end
  end
  
  if (key == "space") and not (state == gamestates[8]) and not (state == gamestates[9]) and not (state == gamestates[10]) and not (state == gamestates[11]) then
    
    if (state == gamestates[7]) then
      state = gamestates[6]
    else
      state = gamestates[7]
    end
  end
end

function love.joystickpressed(joystick,button)
  -- When the right key is pressed.
  if (joystick:isGamepadDown("dpright")) then
    if (player.x == position[1][1]) and (player.y == position[1][2]) then
      DRAWN = 0 -- Position transition will begin
      player.x = position[2][1] -- Player's x position will go right, position table index will go up by 1, leading to a new x position.
      player.y = position[2][2] -- Position table index will go up by 1, leading to a new y position.
      --moveSound:play() -- Play move sound
    elseif (player.x == position[2][1]) and (player.y == position[2][2]) then
      DRAWN = 0 -- Position transition will begin
      player.x = position[3][1] -- Position table index i will go up by 1, leading to new x position.
      player.y = position[3][2] -- Position table index i will go up by 1, leading to new y position.
      --moveSound:play() -- Play move sound
    end
    
    if ((position == pos2) or (position == pos3)) then
      if (player.x == position[3][1]) and (player.y == position[3][2]) then
        DRAWN = 0 -- Position transition will begin
        player.x = position[4][1] -- Position table index i will go up by 1, leading to a new x position
        player.y = position[4][2] -- Position table index i will go up by 1, leading to a new y position
        --moveSound:play() -- Play move sound
      elseif (player.x == position[4][1]) and (player.y == position[4][2]) then
        DRAWN = 0 -- Position transition will begin
        player.x = position[5][1] -- Position table index i will go up by 1, leading to a new x position
        player.y = position[5][2] -- Position table index i will go up by 1, leading to a new y position
        --moveSound:play() -- Play move sound
      end
    end
    
    if ((position == pos3) and not(position == pos2)) then
      if (player.x == position[5][1]) and (player.y == position[5][2]) then
        DRAWN = 0 -- Position transition will begin
        player.x = position[6][1] -- Position table index i will go up by 1, leading to a new x position
        player.y = position[6][2] -- Position table index i will go up by 1, leading to a new y position
        --moveSound:play() -- Play move sound
      elseif (player.x == position[6][1]) and (player.y == position[6][2]) then
        DRAWN = 0 -- Position transition will begin
        player.x = position[7][1] -- Position table index i will go up by 1, leading to a new x position
        player.y = position[7][2] -- Position table index i will go up by 1, leading to a new y position
        --moveSound:play() -- Play move sound
      end
    end
  end
  
  if (joystick:isGamepadDown("dpleft")) then
    if ((position == pos3) and not (position == pos2)) then
      if (player.x == position[7][1]) and (player.y == position[7][2]) then
        DRAWN = 0 -- Position transition will begin
        player.x = position[6][1] -- Position table index i will go down by 1, leading to a new x position
        player.y = position[6][2] -- Position table index i will go down by 1, leading to a new y position
        --moveSound:play() -- Play move sound
      elseif (player.x == position[6][1]) and (player.y == position[6][2]) then
        DRAWN = 0 -- Position transition will begin
        player.x = position[5][1] -- Position table index i will go down by 1, leading to a new x position
        player.y = position[5][2] -- Position table index i will go down by 1, leading to a new y position
        --moveSound:play() -- Play move sound
      end
    end
    
    if ((position == pos2) or (position == pos3)) then
      if (player.x == position[5][1]) and (player.y == position[5][2]) then
        DRAWN = 0 -- Position transition will begin
        player.x = position[4][1] -- Position table index i will go down by 1, leading to a new x position
        player.y = position[4][2] -- Position table index i will go down by 1, leading to a new y position
        --moveSound:play() -- Play move sound
      elseif (player.x == position[4][1]) and (player.y == position[4][2]) then
        DRAWN = 0 -- Position transition will begin
        player.x = position[3][1] -- Position table index i will go down by 1, leading to a new x position
        player.y = position[3][2] -- Position table index i will go down by 1, leading to a new y position
        --moveSound:play() -- Play move sound
      end
    end
    
    if (player.x == position[3][1]) and (player.y == position[3][2]) then
      DRAWN = 0 -- Position transition will begin
      player.x = position[2][1] -- Position table index i will go down by 1, leading to a new x position
      player.y = position[2][2] -- Position table index i will go down by 1, leading to a new y position
      --moveSound:play() -- Play move sound
    elseif (player.x == position[2][1]) and (player.y == position[2][2]) then
      DRAWN = 0 -- Position transition will begin
      player.x = position[1][1] -- Position table index i will go down by 1, leading to a new x position
      player.y = position[1][2] -- Position table index i will go down by 1, leading to a new y position
      --moveSound:play() -- Play move sound
    end
  end
end

function love.mousepressed(x,y,button,istouch)
  if (button == 1) then
    if (mouseX < touchscreenRectangle_left.x + touchscreenRectangle_left.width and mouseX + mouseWidth > touchscreenRectangle_left.x) and (mouseY < touchscreenRectangle_left.y + touchscreenRectangle_left.height and mouseY + mouseHeight > touchscreenRectangle_left.y) then
      if (state == gamestates[6]) then
        if ((position == pos3) and not (position == pos2)) then
          if (player.x == position[7][1]) and (player.y == position[7][2]) then
            DRAWN = 0 -- Position transition will begin
            player.x = position[6][1] -- Position table index i will go down by 1, leading to a new x position
            player.y = position[6][2] -- Position table index i will go down by 1, leading to a new y position
            --moveSound:play() -- Play move sound
          elseif (player.x == position[6][1]) and (player.y == position[6][2]) then
            DRAWN = 0 -- Position transition will begin
            player.x = position[5][1] -- Position table index i will go down by 1, leading to a new x position
            player.y = position[5][2] -- Position table index i will go down by 1, leading to a new y position
            --moveSound:play() -- Play move sound
          end
        end
        
        if ((position == pos2) or (position == pos3)) then
          if (player.x == position[5][1]) and (player.y == position[5][2]) then
            DRAWN = 0 -- Position transition will begin
            player.x = position[4][1] -- Position table index i will go down by 1, leading to a new x position
            player.y = position[4][2] -- Position table index i will go down by 1, leading to a new y position
            --moveSound:play() -- Play move sound
          elseif (player.x == position[4][1]) and (player.y == position[4][2]) then
            DRAWN = 0 -- Position transition will begin
            player.x = position[3][1] -- Position table index i will go down by 1, leading to a new x position
            player.y = position[3][2] -- Position table index i will go down by 1, leading to a new y position
            --moveSound:play() -- Play move sound
          end
        end
        
        if (player.x == position[3][1]) and (player.y == position[3][2]) then
          DRAWN = 0 -- Position transition will begin
          player.x = position[2][1] -- Position table index i will go down by 1, leading to a new x position
          player.y = position[2][2] -- Position table index i will go down by 1, leading to a new y position
          --moveSound:play() -- Play move sound
        elseif (player.x == position[2][1]) and (player.y == position[2][2]) then
          DRAWN = 0 -- Position transition will begin
          player.x = position[1][1] -- Position table index i will go down by 1, leading to a new x position
          player.y = position[1][2] -- Position table index i will go down by 1, leading to a new y position
          --moveSound:play() -- Play move sound
        end
      end
    end
  end
  
  if (mouseX < touchscreenRectangle_right.x + touchscreenRectangle_right.width and mouseX + mouseWidth > touchscreenRectangle_right.x) and (mouseY < touchscreenRectangle_right.y + touchscreenRectangle_right.height and mouseY + mouseHeight > touchscreenRectangle_right.y) then
    if (state == gamestates[6]) then
      if (player.x == position[1][1]) and (player.y == position[1][2]) then
        DRAWN = 0 -- Position transition will begin
        player.x = position[2][1] -- Player's x position will go right, position table index will go up by 1, leading to a new x position.
        player.y = position[2][2] -- Position table index will go up by 1, leading to a new y position.
        --moveSound:play() -- Play move sound
      elseif (player.x == position[2][1]) and (player.y == position[2][2]) then
        DRAWN = 0 -- Position transition will begin
        player.x = position[3][1] -- Position table index i will go up by 1, leading to new x position.
        player.y = position[3][2] -- Position table index i will go up by 1, leading to new y position.
        --moveSound:play() -- Play move sound
      end
      
      if ((position == pos2) or (position == pos3)) then
        if (player.x == position[3][1]) and (player.y == position[3][2]) then
          DRAWN = 0 -- Position transition will begin
          player.x = position[4][1] -- Position table index i will go up by 1, leading to a new x position
          player.y = position[4][2] -- Position table index i will go up by 1, leading to a new y position
          --moveSound:play() -- Play move sound
        elseif (player.x == position[4][1]) and (player.y == position[4][2]) then
          DRAWN = 0 -- Position transition will begin
          player.x = position[5][1] -- Position table index i will go up by 1, leading to a new x position
          player.y = position[5][2] -- Position table index i will go up by 1, leading to a new y position
          moveSound:play() -- Play move sound
        end
      end
      
      if ((position == pos3) and not(position == pos2)) then
        if (player.x == position[5][1]) and (player.y == position[5][2]) then
          DRAWN = 0 -- Position transition will begin
          player.x = position[6][1] -- Position table index i will go up by 1, leading to a new x position
          player.y = position[6][2] -- Position table index i will go up by 1, leading to a new y position
          --moveSound:play() -- Play move sound
        elseif (player.x == position[6][1]) and (player.y == position[6][2]) then
          DRAWN = 0 -- Position transition will begin
          player.x = position[7][1] -- Position table index i will go up by 1, leading to a new x position
          player.y = position[7][2] -- Position table index i will go up by 1, leading to a new y position
          --moveSound:play() -- Play move sound
        end
      end
    end
  end
end


-- Default love update function
function love.update(dt)
  mouseX = love.mouse.getX()
  mouseY = love.mouse.getY()
  --mousePressed(mouseX,mouseY)
  if (state == gamestates[6]) then
    if not (state == gamestates[7]) and not (state == gamestates[8]) and not (state == gamestates[9]) and not (state == gamestates[10]) and not (state == gamestates[11]) then
      if (num <= 0) then
        if (BOOKS_GOTTEN < BOOKS_NEDDED) then
          state = gamestates[10]
        end
      end
      
      if (misses >= 3) then
        state = gamestates[10]
      end
      
      num = num - dt
      if (player.x == position[1][1] and player.y == position[1][2]) then
        playerImage = playerGFXtable[1]
      elseif (player.x == position[2][1] and player.y == position[2][2]) then
        playerImage = playerGFXtable[2] 
      elseif (player.x == position[3][1] and player.y == position[3][2]) then
        playerImage = playerGFXtable[3]
      end
      
      collisionRect.x = player.x -- Collision rectangle x position
      collisionRect.y = player.y -- Collision rectangle y position
      checkCollision(book_table.items,collisionRect)
      checkFailedCollision(book_table.items,failCollisionBoxes)
      -- Following procedure is what is done during the transition between posititons. This helps give the game an even bigger Game & Watch eletronic feel.
      if (DRAWN == 0) then
        notDrawTime = notDrawTime - dt -- The not draw time will be decreased
        
        if (notDrawTime < 0) then -- If the not draw time is less than 0.
          DRAWN = 1 -- DRAWN will be equal to 1, meaning the characters will be drawn in their new positions.
          notDrawTime = notAppearTime -- the not draw time will be equal to the not appear time
        end
      end
      
      for i,v in ipairs(book_table.items) do
        if (v.hasCollided == true) and (#book_table.items > 0) then
          table.remove(book_table.items, i)
        end
      end
      
      for _,v in pairs(book_table.items) do
        -- Following procedure is what is done during the transition between posititons. This helps give the game an even bigger Game & Watch eletronic feel.
        if (v.bookDRAWN == 0) then
          v.bookNotDrawTime = v.bookNotDrawTime - dt -- The not draw time will be decreased
          
          if (v.bookNotDrawTime < 0) then -- If the not draw time is less than 0.
            v.bookDRAWN = 1 -- bookDRAWN will be equal to 1, meaning the characters will be drawn in their new positions.
            v.bookNotDrawTime = v.bookNotAppearTime -- the not draw time will be equal to the not appear time
            v.bookAppearTime = 0.50 -- Set book appearance time back to 1.
            v.y = v.y + 50 -- Increase Position
            
            timeSound:play() -- Play tick sound
        
            if (timeSound:isPlaying() == false) then
              timeSound:stop() -- If the tick is not playing, then stop the sound.
            end
          end
        end
      end
      
      if (BEGIN_ITEM_TIMER == true) then
        itemAppearTime = itemAppearTime - dt
      end
      
      

      
      for _,v in pairs(book_table.items) do 
        v.bookAppearTime = v.bookAppearTime - dt -- Decrase book appearance time
      end
      
      
      for _,v in pairs(book_table.items) do
        if (v.bookAppearTime < 0) then
          v.bookDRAWN = 0
        end
      end
      
      
      if (itemAppearTime < 0) then
        randomAppear = love.math.random(3,5)
        itemAppearTime = randomAppear
      end
      
      if (itemAppearTime >= randomAppear) then
        randomX = love.math.random(1,3)
        book_table:spawn(position[randomX][1] + (player.height/2),0 + 24,bookGFX)
        itemAppearTime = love.math.random(2,5)
        timeSound:play() -- Play tick sound
        BEGIN_ITEM_TIMER = true
        
        if (timeSound:isPlaying() == false) then
          timeSound:stop() -- If the tick is not playing then stop sound
        end
      end
    end
  end
  
  if (state == gamestates[10]) then
    endWait = endWait - dt
    
    if (endWait <= 0) then
      endWait = 3 
      state = gamestates[11]
      --screenState = screenstates[2]
    end
  end
end

-- Default draw function
function love.draw(screen)
  love.graphics.setBackgroundColor(fixColor(245,245,245,255)) -- Set background color to a snowy shade of white.
  love.graphics.getColor(fixColor(255,255,255,255))
  if (state == gamestates[6]) or (state == gamestates[7]) or (state == gamestates[10]) or (state == gamestates[8]) then
    love.graphics.setColor(fixColor(255,255,255,255)) -- Set lower opacity.
    brainGFX:draw(love.graphics.getWidth()/2,love.graphics.getHeight()/2) -- Draw nucleus
    love.graphics.setColor(fixColor(255,255,255,50)) -- Set even lower opacity.
    brainGFX:draw((love.graphics.getWidth()/2) + 5,(love.graphics.getHeight()/2) + 5) -- Draw nucleus
    love.graphics.setColor(fixColor(255,255,255,255)) -- Set color
    local ox,oy = player.width * .5, player.height * .5 -- Set ox oy as the player x and y offset when scailing.
    -- If the DRAWN psuedo bool variable is 1 (true)
    if (DRAWN == 1) then
      playerImage:draw(player.x + ox,player.y + oy) -- Draw the player gfx svg at the x and y position plus it's offsets
      love.graphics.setColor(fixColor(255,255,255,50)) -- Set color again, this time with lower opacity.
      playerImage:draw((player.x + ox) + 5, (player.y + oy)) -- Draw the player gfx svg at the player.x and player.y plus it's offset (add x position plus 5 to add to shadow effect)
      love.graphics.setColor(fixColor(255,255,255,255)) -- Set color again.
    end
    love.graphics.setColor(fixColor(255,255,255,0)) -- Set opacity to 0 since the collision box shouldn't be seen
    love.graphics.rectangle("fill",collisionRect.x, collisionRect.y, collisionRect.width,collisionRect.height) -- Draw box
    love.graphics.setColor(fixColor(255,255,255,255)) -- Set opacity back to 255.
    -- Draw books with shadow and everything
    for _,v in pairs(book_table.items) do
      if (v.bookDRAWN == 1) then
        love.graphics.setColor(fixColor(255,255,255,255)) -- Set color
        bookGFX:draw(v.x + 5,v.y + 5) -- Draw svg
        love.graphics.setColor(fixColor(255,255,255,50)) -- Set color with lower opacity to create shadow
        bookGFX:draw(v.x + 10,v.y + 10) -- Draw svg
        love.graphics.setColor(fixColor(255,255,255,255)) -- Set color back to normal opacity.
      end
    end
    
    love.graphics.setColor(fixColor(255,255,255,0)) -- Set opacity to 0
    for _,v in pairs(failCollisionBoxes) do
      love.graphics.rectangle("fill",v[1],v[2],32,32)
    end
    
    love.graphics.setColor(fixColor(139,69,19,250)) -- Set color to brown
    --love.graphics.rectangle("fill",position[#position][1],0,500,60) -- Draw rectangle (Wall Block)
    love.graphics.rectangle("fill",platform.x,platform.y,platform.width,platform.height) -- Draw rectangle (Platform)
    love.graphics.setColor(fixColor(139,69,19,50)) -- Set color to same color, but with lower opacity.
    --love.graphics.rectangle("fill",(position[#position][1]) + 5,0 + 5,500,60) -- Draw rectangle (Wall Block)
    love.graphics.rectangle("fill",platform.x + 5,platform.y + 5,platform.width,platform.height) -- Draw rectangle (Platform)
    love.graphics.setColor(fixColor(255,255,255,255)) -- Set color
    
    love.graphics.setFont(lcdFont) -- Set font to lcdFont
    love.graphics.setColor(fixColor(0,0,0,255)) -- Set font color to black
    love.graphics.print(math.ceil(num),love.graphics.getWidth()/2 - lcdFont:getWidth(tostring(math.ceil(num)))/2,love.graphics.getHeight() - 70) -- Draw score using lcdfont
    love.graphics.setColor(fixColor(0,0,0,50)) -- Set font color to black, but with lower opacity.
    love.graphics.print(math.ceil(num),(love.graphics.getWidth()/2 - lcdFont:getWidth(tostring(math.ceil(num)))/2) + 5,(love.graphics.getHeight() - 70) + 5) -- Draw score using lcdfont
    love.graphics.setColor(fixColor(255,255,255,255)) -- Set font color
    
    love.graphics.setColor(fixColor(0,0,0,255)) -- Set font color to black
    love.graphics.print(tostring(score),70, love.graphics.getHeight() - 70) -- Print score
    love.graphics.print(tostring(BOOKS_GOTTEN),love.graphics.getWidth() - 100,love.graphics.getHeight() - 70) -- Print books gotten
    love.graphics.setFont(arcoFontSmall) -- Set font to arco
    love.graphics.print("Books",love.graphics.getWidth() - 120, love.graphics.getHeight() - 30) -- Print book
    love.graphics.print("Score",50,love.graphics.getHeight() - 30)
    love.graphics.setColor(fixColor(0,0,0,50)) -- Set font color with lower opacity.
    love.graphics.setFont(lcdFont) -- Set font to lcdFont
    love.graphics.print(tostring(score),70 + 5, (love.graphics.getHeight() - 70) + 5)-- Print score
    love.graphics.print(tostring(BOOKS_GOTTEN),(love.graphics.getWidth() - 100) + 5,(love.graphics.getHeight() - 70) + 5) -- Print books gotten
    love.graphics.setFont(arcoFontSmall) -- Set font
    love.graphics.print("Books",(love.graphics.getWidth() - 120) + 5, (love.graphics.getHeight() - 30) + 5) -- Print books
    love.graphics.print("Score",50 + 5,(love.graphics.getHeight() - 30) + 5) -- Print score
    love.graphics.setColor(fixColor(0,0,0,255)) -- Set color to white
    
    if (misses >= 1) then -- If the player has drawn missed 1 or more books
      love.graphics.rectangle("fill",errorBox.gfxes[1][1],errorBox.gfxes[1][2],errorBox.width,errorBox.height) -- Draw miss box
    end
    
    if (misses >= 2) then -- If the player has missed 2 or more books
      love.graphics.rectangle("fill",errorBox.gfxes[2][1],errorBox.gfxes[2][2],errorBox.width,errorBox.height) -- Draw miss box
    end
    
    if (misses >= 3) then -- If the player has missed 3 or more books
      love.graphics.rectangle("fill",errorBox.gfxes[3][1],errorBox.gfxes[3][2],errorBox.width,errorBox.height) -- Draw miss box
    end
    
    
    
    love.graphics.setColor(fixColor(0,0,0,255))

    
    
    love.graphics.rectangle("line",love.graphics.getWidth() - 24, 16,8,24) -- Draw knowledge meter box
    if ((BOOKS_GOTTEN >= (BOOKS_NEDDED - 10)) and (BOOKS_GOTTEN < BOOKS_NEDDED))  then -- If the books gotten is less than books needed and equal to books needed - 10
      love.graphics.rectangle("fill",love.graphics.getWidth() - 24,16,8,16) -- Partially fill knowledge meter box.
    elseif ( BOOKS_GOTTEN >= BOOKS_NEDDED) then -- If books gotten is greater than or equal to books needed
      love.graphics.rectangle("fill",love.graphics.getWidth() - 24,16,8,24) -- Fully fill meter box.
    end
    love.graphics.setColor(fixColor(0,0,0,50)) -- Set color with lower opacity.
    love.graphics.rectangle("line",(love.graphics.getWidth() - 24) + 5, 16 + 5,8,24)  -- Draw knowledge meter box
    if ((BOOKS_GOTTEN >= (BOOKS_NEDDED - 10)) and (BOOKS_GOTTEN < BOOKS_NEDDED))  then -- If the books gotten is less than books needed and equal to books needed - 10
      love.graphics.rectangle("fill",(love.graphics.getWidth() - 24) + 5,16 + 5,8,16) -- Partially fill knowledge meter box.
    elseif ( BOOKS_GOTTEN >= BOOKS_NEDDED) then -- If books gotten is greater than or equal to books needed
      love.graphics.rectangle("fill",(love.graphics.getWidth() - 24) + 5,16 + 5,8,24) -- Fully fill meter box.
    end
    love.graphics.setColor(fixColor(255,255,255,255)) -- Set opacity and color to normal
    
    love.graphics.setColor(fixColor(205,92,92,0))
    love.graphics.rectangle("fill",touchscreenRectangle_left.x,touchscreenRectangle_left.y,     touchscreenRectangle_left.width,touchscreenRectangle_left.height) -- Draw left rectangle touchscreen box
    love.graphics.setColor(fixColor(21,67,96,0))
    love.graphics.rectangle("fill",touchscreenRectangle_right.x,touchscreenRectangle_right.y,touchscreenRectangle_right.width,touchscreenRectangle_right.height) -- Draw right rectangle touchscren box
    love.graphics.setColor(fixColor(255,255,255,255))
  end
  
  if (state == gamestates[10]) then
    love.graphics.setFont(arcoFontLarge)
    love.graphics.setColor(fixColor(0,0,0,255))
    love.graphics.print("YOU FAILED!", love.graphics.getWidth()/2 - arcoFontLarge:getWidth("YOU FAILED!") / 2, love.graphics.getHeight() / 2 - arcoFontLarge:getHeight("YOU FAILED!")/2)
    love.graphics.setColor(fixColor(0,0,0,50))
    love.graphics.print("YOU FAILED!", love.graphics.getWidth()/2 - arcoFontLarge:getWidth("YOU FAILED!") / 2 + 5, love.graphics.getHeight() / 2 - arcoFontLarge:getHeight("YOU FAILED!")/2 + 5)
    love.graphics.setColor(fixColor(0,0,0,255))
  end
  
  if (state == gamestates[11]) then
    love.graphics.setFont(arcoFontLarge)
    love.graphics.setColor(fixColor(0,0,0,255))
    love.graphics.print("YOU FAILED THE TEST!",love.graphics.getWidth() /2  - arcoFontLarge:getWidth("YOU FAILED THE TEST!") / 2, 50)
    fail_GFX:draw(love.graphics.getWidth()/2,love.graphics.getHeight()/2)
    love.graphics.setColor(fixColor(0,0,0,50))
    love.graphics.print("YOU FAILED THE TEST!",love.graphics.getWidth() /2  - arcoFontLarge:getWidth("YOU FAILED THE TEST!") / 2 + 5, 50 + 5)
    fail_GFX:draw(love.graphics.getWidth()/2 + 5,love.graphics.getHeight()/2 + 5)
    love.graphics.setColor(fixColor(255,255,255,255))
  end
    
  if (state == gamestates[9]) then
    love.graphics.setFont(arcoFontLarge)
    love.graphics.setColor(fixColor(0,0,0,255))
    love.graphics.print("YOU PASSED THE TEST!",love.graphics.getWidth() /2  - arcoFontLarge:getWidth("YOU PASSED THE TEST!") / 2, 50)
    success_GFX:draw(love.graphics.getWidth()/2,love.graphics.getHeight()/2)
    love.graphics.setColor(fixColor(0,0,0,50))
    love.graphics.print("YOU PASSED THE TEST!",love.graphics.getWidth() /2  - arcoFontLarge:getWidth("YOU PASSED THE TEST!") / 2 + 5, 50 + 5)
    success_GFX:draw(love.graphics.getWidth()/2 + 5,love.graphics.getHeight()/2 + 5)
    love.graphics.setColor(fixColor(255,255,255,255))
  end
end

